from sense_push.util import *
from sense_push.common import *
import sense_core as sd


class PushDataHandler(object):

    def __init__(self, database=None, table=None, queue_configs=None):
        self.database = database
        self.table = table
        self.queue_configs = queue_configs

    def start_rebuild_table(self):
        message = build_push_message(PUSH_ACTION_REBUILD_START, self.database, self.table, {})
        self.push_queue_message(message)

    def end_rebuild_table(self):
        message = build_push_message(PUSH_ACTION_REBUILD_END, self.database, self.table, {})
        self.push_queue_message(message)

    def drop_table(self):
        message = build_push_message(PUSH_ACTION_DROP, self.database, self.table, {})
        self.push_queue_message(message)

    def insert_data(self, data):
        message = build_push_message(PUSH_ACTION_INSERT, self.database, self.table, data)
        self.push_queue_message(message)

    def update_data(self, data):
        message = build_push_message(PUSH_ACTION_UPDATE, self.database, self.table, data)
        self.push_queue_message(message)

    def delete_data(self, data):
        message = build_push_message(PUSH_ACTION_DELETE, self.database, self.table, data)
        self.push_queue_message(message)

    def push_queue_message(self, message):
        for config in self.queue_configs:
            producer = sd.RabbitProducer(config_info=config)
            producer.send_safely(config['queue'], sd.dump_json(message), True, 1000000)



from sense_push.util import *
from sense_push.common import *
import sense_core as sd
import pymysql
from sense_push.core.data import PushDataHandler
import datetime

class SensePushManager(object):

    def __init__(self, queue_label=None, db_label=None, queue_configs=None, db_config=None, database=None, queue=None):

        if not db_config:
            db_config = dict()
        if database:
            db_config['database'] = database
        if not queue_configs:
            queue_config = dict()
            queue_config['queue'] = queue
            queue_configs = [
                self._build_config(queue_label, queue_config, ['host', 'port', 'user', 'pass', 'password'])]
        self.db_config = self._build_config(db_label, db_config,
                                            ['host', 'port', 'user', 'pass', 'database', 'password'])
        self.queue_configs = queue_configs

    def _build_sql(self, table, build_sql, offset, limit):
        if build_sql is not None:
            return build_sql(table, offset, limit)
        if limit <= 0:
            return "select * from {}".format(table)
        return "select * from {0} limit {1},{2}".format(table, offset, limit)

    def _build_config(self, label, config, keys):
        if not label:
            return config
        if not config:
            config = dict()
        for key in keys:
            if key in config:
                continue
            config[key] = sd.config(label, key, '')
        return config

    def push_table_all_data(self, table, build_sql=None, limit=0):
        offset = 0
        queue_configs = self.queue_configs
        db_config = self.db_config
        handler = PushDataHandler(database=db_config['database'], table=table, queue_configs=queue_configs)
        while True:
            sql = self._build_sql(table, build_sql, offset, limit)
            items = self.fetch_by_sql(db_config, sql)
            sd.log_info("size={0} for sql={1}".format(len(items), sql))
            for item in items:
                for key in item.keys():
                    value = item[key]
                    if type(value) == datetime.datetime:
                        item[key] = str(value)
                handler.insert_data(item)
            if len(items) < limit or limit <= 0:
                break
            offset += limit
        sd.log_info("done push_table_all_data for {0}".format(table))

    def fetch_by_sql(self, config, sql):
        conn = pymysql.connect(host=config['host'], user=config['user'],
                               password=config['pass'],
                               charset='utf8', database=config['database'],
                               port=int(config['port']))
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute(sql)
        items = cursor.fetchall()
        cursor.close()
        conn.close()
        return items



import sense_core as sd
import logging
from sense_push.common import PUSH_ACTION_UPDATE


def init_config(module='sense_push', module_name=''):
    sd.log_init_config(module, sd.config('log_path'), monit_queue='rabbit_monit', use_module_name=module_name)
    logging.getLogger("pika").propagate = False


def build_push_message(action, database, table, data):
    if action == PUSH_ACTION_UPDATE:
        data = data.get('after')
        if not data:
            sd.log_error("build_push_message failed with table={0} data={1}".format(table, data))
            return None
    return dict(
        action=action,
        db=database,
        table=table,
        data=data,
    )



import sense_core as sd
from django.db.utils import IntegrityError
from sense_push.common import *


class DjangoPushDataHandler(object):

    def __init__(self, model_map=None, models=None):
        if model_map:
            self.model_map = self._convert_model_map(model_map)
        else:
            self.model_map = self._build_model_map(models)
        self.model_field_map = dict()

    def add_model_config(self, model, primary=None, fields=None, exclude_fields=None,table=None):
        if not primary:
            primary = ['id']
        config = {
            'model': model,
            'primary': primary,
        }
        if fields:
            config['fields'] = fields
        if exclude_fields:
            config['exclude_fields'] = exclude_fields
        if not table:
            table = model._meta.db_table
        self.model_map[table] = config

    def _build_model_map(self, models):
        result = dict()
        for model in models:
            result[model._meta.db_table] = {
                'model': model,
                'primary': ['id'],
            }
        return result

    def _convert_model_map(self, model_map):
        result = dict()
        for k, v in model_map.items():
            if type(v) != dict:
                result[k] = {
                    'model': v,
                }
            else:
                result[k] = v
            primary = v.get('primary')
            if not primary:
                v['primary'] = ['id']
        if sd.is_debug():
            sd.log_info("model_map={}".format(result))
        return result

    def get_model_key(self, database, table):
        return "{0}_{1}".format(database, table)

    def get_model_fields(self, model):
        fields = self.model_field_map.get(model)
        if fields:
            return fields
        fields = set([field.name for field in model._meta.get_fields()])
        self.model_field_map[model] = fields
        return fields

    def _get_model(self, database, table):
        key = self.get_model_key(database, table)
        model = self.model_map.get(key)
        if model:
            return model
        return self.model_map.get(table)

    def insert(self, database, table, data):
        self._insert0(database, table, data, "insert")
        self.callback_handle(PUSH_ACTION_INSERT, database, table, data)

    def before_update(self, database, table, data):
        return True

    def _insert0(self, database, table, data, action):
        try:
            model_item = self._get_model(database, table)
            if not model_item:
                return
            if not self.before_update(database, table, data):
                return
            model = model_item['model']
            fields = model_item.get('fields')
            if not fields:
                fields = self.get_model_fields(model)
            else:
                fields = set(fields)
            exclude_fields = model_item.get('exclude_fields')
            if exclude_fields:
                for field in exclude_fields:
                    fields.discard(field)
            data2 = self._build_update_data(data, model_item, fields)
            model.objects.update_or_create(**data2)
            sd.log_info(action + " data table={0} data={1}".format(table, data2))
        except IntegrityError as ex:
            sd.log_warn("IntegrityError for table={0} data={1}".format(table, data))
        except Exception as ex:
            sd.log_exception(ex)
            sd.log_error("execption for table={0} data={1}".format(table, data))
            raise ex

    def update(self, database, table, data):
        after = data.get('after')
        if after:
            data = after
        self._insert0(database, table, data, "update")
        self.callback_handle(PUSH_ACTION_UPDATE, database, table, data)

    def _build_update_data(self, data, model_item, fields):

        res = self.build_primary_keys(data, model_item['primary'])
        if not res:
            sd.log_error("build_primary_keys failed for data={0} primary={1}".format(data, model_item['primary']))
            return None
        data = self.build_field_keys(data, fields)
        res = self.build_update_params(res, data)
        return res

    def build_update_params(self, res, data):
        res['defaults'] = data
        return res

    def build_field_keys(self, data, fields):
        res = {}
        for key, val in data.items():
            if key not in fields:
                continue
            res[key] = val
        return res

    def build_primary_keys(self, data, primary_keys):
        res = {}
        for key in primary_keys:
            val = data.pop(key)
            if val is None:
                return None
            res[key] = val
        return res

    def before_delete(self, database, table, data):
        return True

    def delete(self, database, table, data):
        model_item = self._get_model(database, table)
        if not model_item:
            return
        if not self.before_delete(database, table, data):
            return
        model = model_item['model']
        res = self.build_primary_keys(data, model_item['primary'])
        if not res:
            sd.log_error("build_primary_keys failed for data={0} primary={1}".format(data, model_item['primary']))
            return
        model.objects.filter(**res).delete()
        self.callback_handle(PUSH_ACTION_DELETE, database, table, data)

    def callback_handle(self, action, database, table, data):
        pass



import sense_core as sd
from sense_push.common import *


class SensePushConsumer(object):

    def __init__(self, handler, table=None):
        super(SensePushConsumer, self).__init__()
        self.table = table
        self.handler = handler

    @sd.try_catch_exception
    def consume_queue(self, label=None, config_info=None, queue=None):
        sd.log_info("start consume_queue {}".format(queue))
        consumer = sd.RabbitConsumer2(topic=queue, label=label, config_info=config_info)
        consumer.consume_loop(self.handle_message)

    def handle_message(self, message):
        message2 = sd.load_json(message)
        if not message2:
            sd.log_error("invalid message for {}".format(message))
            return
        table = message2.get('table')
        if self.table and self.table != table:
            return
        sd.log_info("start _handle_message0 {0}".format(message))
        action = message2.get('action')
        database = message2.get('db')
        data = message2.get('data')
        if action == PUSH_ACTION_INSERT:
            self.insert(database=database, table=table, data=data)
        elif action == PUSH_ACTION_UPDATE:
            after = data.get('after')
            if after:
                data = after
            self.update(database=database, table=table, data=data)
        elif action == PUSH_ACTION_DELETE:
            self.delete(database=database, table=table, data=data)
        else:
            sd.log_warn("invalid message {}".format(message))

    def insert(self, database, table, data):
        self.handler.insert(database, table, data)

    def update(self, database, table, data):
        self.handler.update(database, table, data)

    def delete(self, database, table, data):
        self.handler.delete(database, table, data)



from sense_push.common import *


class DumbPushDataHandler(object):

    def insert(self, database, table, data):
        self.callback_handle(PUSH_ACTION_INSERT, database, table, data)

    def update(self, database, table, data):
        self.callback_handle(PUSH_ACTION_UPDATE, database, table, data)

    def delete(self, database, table, data):
        self.callback_handle(PUSH_ACTION_DELETE, database, table, data)

    def callback_handle(self, action, database, table, data):
        pass



import os
from setuptools import setup

requirements = [
    "sense_core>=0.2.61",
    "django"
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='sense-push',
    version='0.1.3',
    packages=[
        "sense_push",
        "sense_push.core",
        "sense_push.common",
        "sense_push.util",
    ],
    license='BSD License',  # example license
    description='sense push',
    install_requires=requirements,
    long_description_content_type="text/markdown",
    url='',
    author='kafka0102',
    author_email='yujianjia@sensedeal.ai',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
)
